unit PlayerUtils;

interface

uses
    Windows, SysUtils, Classes, Controls, Variants,
    ActiveX, OleCtrls, WMPLib_TLB, RealAudioObjects_TLB;

type
    TPlayerCore = class;
    TOnMediaChange = procedure(Sender: TPlayerCore; const Title: String;
                                const Width, Height: Integer) of object;
    TOnStatusChange = procedure(Sender: TPlayerCore; const Status: String) of object;
    TOnOpenStateChange = procedure(Sender: TPlayerCore; const Status: String) of object;
    TOnPlayStateChange = procedure(Sender: TPlayerCore; const Status: String) of object;

    TPlayerCore = class(TObject)
    protected
        //  vC[̃Rei
        FContainer: TWinControl;                                   
        //  \[Xݒ
        procedure SetSource(const AValue: String); virtual; abstract;
        //  \[X擾
        function GetSource(): String; virtual; abstract;
        //  ~[g
        procedure SetMute(const AValue: Boolean); virtual; abstract;
        //  ~[g(̓{ꐳH)
        function GetMute(): Boolean; virtual; abstract;
        //  {[ύX
        procedure SetVolume(const AValue: Integer); virtual; abstract;
        //  {[擾
        function GetVolume(): Integer; virtual; abstract;
        //  oXύX
        procedure SetBalance(const AValue: Integer); virtual; abstract;
        //  oX擾
        function GetBalance(): Integer; virtual; abstract;
        //  Đxݒ
        procedure SetRate(const AValue: Double); virtual; abstract;
        //  Đx擾
        function GetRate(): Double; virtual; abstract;
        //  ^Cg擾
        function GetTitle(): String; virtual; abstract;
    public
        //  RXgN^
        constructor Create(const AContainer: TWinControl); virtual;
        //  fXgN^
        destructor Destroy(); override;
    public
        //  ContainerJ
        property Container: TWinControl read FContainer;
        //  \[X
        property Source: String read GetSource write SetSource;
        //  ~[g
        property Mute: Boolean read GetMute write SetMute;
        //  {[
        property Volume: Integer read GetVolume write SetVolume;
        //  Balance
        property Balance: Integer read GetBalance write SetBalance;
        //  Đx
        property Rate: Double read GetRate write SetRate;
        //  ^Cg
        property Title: String read GetTitle;
        //  fBAǂݍł邩
        function MediaExists(): Boolean; virtual; abstract;
        //  V[N(b)
        procedure SeekFromBeginning(const Seconds: Double); virtual; abstract;
        procedure SeekFromCurrent(const Seconds: Double); virtual; abstract;
        procedure SeekFromEnd(const Seconds: Double); virtual; abstract;
        //  V[N()
        procedure SeekFromBeginning2(const Percentage: Double);
        procedure SeekFromCurrent2(const Percentage: Double);
        //  Đ
        procedure Play(); virtual; abstract;
        //  ꎞ~
        procedure Pause(); virtual; abstract;
        //  ~
        procedure Stop(); virtual; abstract;
        //  TCY
        procedure OnResizeForm(); virtual; abstract;
        //  Đ/ꎞ~؂ւ
        procedure PlayOrPause(); virtual; abstract;
        //  ݈ʒu
        function CurrentPosition(): Double; virtual; abstract;
        //  
        function MediaLength(): Double; virtual; abstract;
        //  ݈ʒu(b)
        function CurrentPositionHMS(): String;
        //  (b)
        function MediaLengthHMS(): String;
    public
        //  fBAύX
        OnMediaChange: TOnMediaChange;
        //  Xe[^XύX
        OnStatusChange: TOnStatusChange;
        //  fBAI[vXe[^XύX
        OnOpenStateChange: TOnOpenStateChange;
        //  fBAvCXe[^XύX
        OnPlayStateChange: TOnPlayStateChange;
        //  fobOp
        function GetDebugState(): Integer; virtual; abstract;
    end;

type
    TWmpPlayer = class(TPlayerCore)
    private
        FPlayer: TWindowsMediaPlayer;
        procedure StrechToFit();
    public
        constructor Create(const AContainer: TWinControl); override;
        destructor Destroy(); override;
    protected
        procedure SetSource(const AValue: String); override;
        function GetSource(): String; override;
        procedure SetMute(const AValue: Boolean); override;
        function GetMute(): Boolean; override;
        procedure SetVolume(const AValue: Integer); override;
        function GetVolume(): Integer; override;
        procedure SetBalance(const AValue: Integer); override;
        function GetBalance(): Integer; override;
        procedure SetRate(const AValue: Double); override;
        function GetRate(): Double; override;
        function GetTitle(): String; override;
    protected
        procedure MediaChange(Sender: TObject; const Item: IDispatch);
        procedure OpenStateChange(Sender: TObject; NewState: Integer);
        procedure PlayStateChange(Sender: TObject; NewState: Integer);
        procedure StatusChange(Sender: TObject);
    public
        function MediaExists(): Boolean; override;
        procedure SeekFromBeginning(const Seconds: Double); override;
        procedure SeekFromCurrent(const Seconds: Double); override;
        procedure SeekFromEnd(const Seconds: Double); override;
        procedure Play(); override;
        procedure Pause(); override;
        procedure Stop(); override;
        procedure OnResizeForm(); override;
        procedure PlayOrPause(); override;
        function CurrentPosition(): Double; override;
        function MediaLength(): Double; override;
        function GetDebugState(): Integer; override;
    end;

type
    TRealPlayer = class(TPlayerCore)
    private
        FPlayer: TRealAudio;
        FFirst: Boolean;
        procedure StrechToFit();
    public
        constructor Create(const AContainer: TWinControl); override;
        destructor Destroy(); override;
    protected
        procedure SetSource(const AValue: String); override;
        function GetSource(): String; override;
        procedure SetMute(const AValue: Boolean); override;
        function GetMute(): Boolean; override;
        procedure SetVolume(const AValue: Integer); override;
        function GetVolume(): Integer; override;
        procedure SetBalance(const AValue: Integer); override;
        function GetBalance(): Integer; override;
        procedure SetRate(const AValue: Double); override;
        function GetRate(): Double; override;
        function GetTitle(): String; override;
    protected
        procedure ClipOpened(Sender: TObject; const shortClipName, url: WideString);
        procedure ShowStatus(Sender: TObject; const statusText: WideString);
    public
        function MediaExists(): Boolean; override;
        procedure SeekFromBeginning(const Seconds: Double); override;
        procedure SeekFromCurrent(const Seconds: Double); override;
        procedure SeekFromEnd(const Seconds: Double); override;
        procedure Play(); override;
        procedure Pause(); override;
        procedure Stop(); override;
        procedure OnResizeForm(); override;
        procedure PlayOrPause(); override;
        function CurrentPosition(): Double; override;
        function MediaLength(): Double; override;
        function GetDebugState(): Integer; override;
    end;

function SetMasterVolume(const MixerID, DstID: Integer; const Value: WORD): Boolean;

implementation

uses
    MMSystem;

//=============================================================================
//  Utility
//=============================================================================

//-----------------------------------------------------------------------------
//  http://delfusa.main.jp/delfusafloor/archive/www.nifty.ne.jp_forum_fdelphi/samples/00507.html
//  PUԉcuʐ΍݂ȂōSamplevɊ񂹂ꂽTv
//  {[iʁjς|Ŝ̃{[ς
//  SetVolume: ~LT[̃~bNXς݃{[ݒ肷
//  MixerID: ~LT[̂hcimixerGetNumDevs QƁj
//  DstID: ~bNXσC̔ԍimixerGetDevCaps QƁj
//  Value: {[A0-65535($FFFF)
function SetMasterVolume(const MixerID, DstID: Integer; const Value: WORD): Boolean;
var
    mmr: MMRESULT;
    DstLine: TMixerLine;
    MixLineCtrl: TMixerLineControls;
    MixCtrl: TMixerControl;
    MixCtrlDet: TMixerControlDetails;
    i: Integer;
    Vol: Pointer;
begin
    //  TMixerLine
    ZeroMemory(@DstLine, SizeOf(DstLine));
    DstLine.cbStruct := 1024;
    DstLine.dwDestination := DstID;
    //  mixerGetLineInfo
    mmr := mixerGetLineInfo(MixerID, @DstLine, MIXER_GETLINEINFOF_DESTINATION or MIXER_OBJECTF_MIXER);
    if (mmr = MMSYSERR_NOERROR) then
    begin
        //  TMixerLineControls
        ZeroMemory(@MixLineCtrl, SizeOf(MixLineCtrl));
        MixLineCtrl.cbStruct := SizeOf(TMixerLineControls);
        MixLineCtrl.cbmxctrl := SizeOf(TMixerControl);
        MixLineCtrl.dwLineID := DstLine.dwLineID;
        MixLineCtrl.dwControlType := MIXERCONTROL_CT_CLASS_FADER or MIXERCONTROL_CONTROLTYPE_VOLUME;
        MixLineCtrl.pamxctrl := @MixCtrl;
        MixLineCtrl.cControls := 1;
        //  TMixerControl
        ZeroMemory(@MixCtrl, SizeOf(MixCtrl));
        MixCtrl.cbStruct := SizeOf(TMixerControl);
        //  mixerGetLineControls
        mmr := mixerGetLineControls(MixerID, @MixLineCtrl, MIXER_GETLINECONTROLSF_ONEBYTYPE or MIXER_OBJECTF_MIXER);
        if (mmr = MMSYSERR_NOERROR) then
        begin
            //  TMixerControlDetails
            ZeroMemory(@MixCtrlDet, SizeOf(MixCtrlDet));
            MixCtrlDet.cbStruct := SizeOf(TMixerControlDetails);
            MixCtrlDet.dwControlID := MixCtrl.dwControlID;
            MixCtrlDet.cChannels := DstLine.cChannels;
            MixCtrlDet.cbDetails := SizeOf(TMixerControlDetailsUnsigned);
            //  mixerSetControlDetails
            GetMem(Vol, MixCtrlDet.cChannels * SizeOf(DWORD));
            try
                MixCtrlDet.paDetails := Vol;
                for i := 0 to MixCtrlDet.cChannels - 1 do
                begin
                    PDWORD(Integer(Vol) + SizeOf(DWORD) * i)^ := value;
                end;
                mmr := mixerSetControlDetails(MixerID, @MixCtrlDet, MIXER_OBJECTF_MIXER or MIXER_SETCONTROLDETAILSF_VALUE);
            finally
              FreeMem(Vol);
            end;
        end
        else
            //  ERROR of 'mixerGetLineControls'
        ;
    end
    else
        //  ERROR of 'mixerGetLineInfo'
    ;

    Result := mmr = MMSYSERR_NOERROR;
end;


//=============================================================================
//  TPlayer implementation
//=============================================================================

//-----------------------------------------------------------------------------
constructor TPlayerCore.Create(const AContainer: TWinControl);
begin
    inherited Create();
    FContainer := AContainer;
end;
//-----------------------------------------------------------------------------
destructor TPlayerCore.Destroy();
begin
    inherited;
end;
//-----------------------------------------------------------------------------
//  @bHH:MM:SS \
function SecondToText(const ASecond: Integer): String;
var
    LTemp: Integer;
    LHour: Integer;
    LMinute: Integer;
    LSecond: Integer;
begin
    LTemp := ASecond;
    //  1M = 60SEC
    //  1H = 60 * 60 = 3600SEC
    LSecond := LTemp mod 60;
    LTemp := (LTemp - LSecond) div 60;
    LMinute := LTemp mod 60;
    LTemp := (LTemp - LMinute) div 60;
    LHour := LTemp;
    if (LHour = 0) then
    begin
        Result := Format('%2.2d:%2.2d', [LMinute, LSecond]);
    end
    else
    begin
        Result := Format('%2.2d:%2.2d:%2.2d', [LHour, LMinute, LSecond]);
    end;
end;
//-----------------------------------------------------------------------------
function TPlayerCore.CurrentPositionHMS(): String;
begin
    Result := SecondToText(Trunc(CurrentPosition()));
end;
//-----------------------------------------------------------------------------
function TPlayerCore.MediaLengthHMS(): String;
begin
    Result := SecondToText(Trunc(MediaLength()));
end;
//-----------------------------------------------------------------------------
procedure TPlayerCore.SeekFromBeginning2(const Percentage: Double);
var
    LLength: Double;
begin
    LLength := Self.MediaLength;
    SeekFromBeginning(LLength * Percentage / 100);
end;
//-----------------------------------------------------------------------------
procedure TPlayerCore.SeekFromCurrent2(const Percentage: Double);
var
    LLength: Double;
begin
    LLength := Self.MediaLength;
    SeekFromCurrent(LLength * Percentage / 100);
end;

//=============================================================================
//  TWmpPlayer implementation
//=============================================================================

//-----------------------------------------------------------------------------
procedure TWmpPlayer.StrechToFit();
var
    LCore: IWMPCore;
    LPlayer4Disp: WMPLib_TLB.IWMPPlayer4Disp;
begin
    //  vC[̃C^[tF[Xo(?)
    LCore := FPlayer.ControlInterface;
    if Assigned(LCore) then
    begin
        //  StretchToFit ĂC^[tF[Xւ̎QƂo
        LCore.QueryInterface(WMPLib_TLB.IWMPPlayer4Disp, LPlayer4Disp);
        if Assigned(LPlayer4Disp) then
        begin
            //  ŁAݒ肷
            LPlayer4Disp.StretchToFit := True;
            //  Đ̈悪LȂ킯Ȃ̂ł߂Ă
            //LPlayer4Disp.uiMode := 'mini';
        end;
    end;
end;

//-----------------------------------------------------------------------------
constructor TWmpPlayer.Create(const AContainer: TWinControl);
begin
    inherited;
    //  CX^X쐬
    FPlayer := TWindowsMediaPlayer.Create(nil);
    //  ĐOFF
    FPlayer.settings.autoStart := False;
    //  ʃTCYςɍĐ
    StrechToFit();
    //  \EBhEݒ
    FPlayer.Parent := AContainer;
    FPlayer.Align := alClient;
    FPlayer.TabStop := False;
    //  Cxgݒ
    FPlayer.OnMediaChange := Self.MediaChange;
    FPlayer.OnStatusChange := Self.StatusChange;
    FPlayer.OnPlayStateChange := Self.PlayStateChange;
    FPlayer.OnOpenStateChange := Self.OpenStateChange;
end;
//-----------------------------------------------------------------------------
destructor TWmpPlayer.Destroy();
begin
    FPlayer.Free();
    inherited;
end;
//-----------------------------------------------------------------------------
procedure TWmpPlayer.SetSource(const AValue: String);
begin
    FPlayer.URL := AValue;
end;
//-----------------------------------------------------------------------------
function TWmpPlayer.GetSource(): String;
begin
    Result := FPlayer.URL;
end;
//-----------------------------------------------------------------------------
function TWmpPlayer.MediaExists(): Boolean;
begin
    Result := (FPlayer.currentMedia <> nil);
end;
//-----------------------------------------------------------------------------
procedure TWmpPlayer.SetMute(const AValue: Boolean); 
begin
    FPlayer.settings.mute := AValue;
end;
//-----------------------------------------------------------------------------
function TWmpPlayer.GetMute(): Boolean;
begin
    Result := FPlayer.settings.mute;
end;
//-----------------------------------------------------------------------------
procedure TWmpPlayer.SetVolume(const AValue: Integer);
begin
    FPlayer.settings.volume := AValue;
end;
//-----------------------------------------------------------------------------
function TWmpPlayer.GetVolume(): Integer;
begin
    Result := FPlayer.settings.volume;
end;
//-----------------------------------------------------------------------------
procedure TWmpPlayer.SetBalance(const AValue: Integer);
begin
    FPlayer.settings.balance := AValue;
end;
//-----------------------------------------------------------------------------
function TWmpPlayer.GetBalance(): Integer;
begin
    Result := FPlayer.settings.balance;
end;
//-----------------------------------------------------------------------------
procedure TWmpPlayer.SetRate(const AValue: Double);
begin
    FPlayer.settings.rate := AValue;
end;
//-----------------------------------------------------------------------------
function TWmpPlayer.GetRate(): Double;
begin
    Result := FPlayer.settings.rate;
end;
//-----------------------------------------------------------------------------
function TWmpPlayer.GetTitle(): String;
begin
    if MediaExists() then
    begin
        Result := FPlayer.currentMedia.name;
    end
    else
    begin
        Result := '';
    end;
end;
//-----------------------------------------------------------------------------
procedure TWmpPlayer.SeekFromBeginning(const Seconds: Double);
begin
    if MediaExists() then
    begin
        FPlayer.controls.currentPosition := Seconds;
    end;
end;
//-----------------------------------------------------------------------------
procedure TWmpPlayer.SeekFromCurrent(const Seconds: Double);
begin
    if MediaExists() then
    begin
        SeekFromBeginning(FPlayer.controls.currentPosition + Seconds);
    end;
end;
//-----------------------------------------------------------------------------
procedure TWmpPlayer.SeekFromEnd(const Seconds: Double);
begin
    if MediaExists() then
    begin
        SeekFromBeginning(FPlayer.currentMedia.duration - Seconds);
    end;
end;
//-----------------------------------------------------------------------------
procedure TWmpPlayer.Play();
begin
    if MediaExists() then FPlayer.controls.play();
end;
//-----------------------------------------------------------------------------
procedure TWmpPlayer.Pause();
begin
    if MediaExists() then FPlayer.controls.pause();
end;
//-----------------------------------------------------------------------------
procedure TWmpPlayer.Stop();
begin
    if MediaExists() then FPlayer.controls.stop();
end;
//-----------------------------------------------------------------------------
//  tH[TCYɒǏ]
//  BBS ւ̏(2007/12 ern )
procedure TWmpPlayer.OnResizeForm();
const
    IID_IOleInPlaceObject: TGUID = '{00000113-0000-0000-C000-000000000046}';
var
    IOIPObj: IOleInPlaceObject;
    WMPRect: TRect;
begin
    SetRect(WMPRect, 0, 0, FContainer.Width, FContainer.Height);
    IDispatch(FPlayer.OleObject).QueryInterface(IID_IOleInPlaceObject, IOIPObj);
    if Assigned(IOIPObj) then
    begin
        IOIPObj.SetObjectRects(WMPRect, WMPRect);
    end;
end;
//-----------------------------------------------------------------------------
procedure TWmpPlayer.PlayOrPause();
begin
    if (FPlayer.controls.isAvailable['Play']) then
    begin
        FPlayer.controls.play();
    end
    else
    if (FPlayer.controls.isAvailable['Pause']) then
    begin
        FPlayer.controls.pause();
    end;
end;
//-----------------------------------------------------------------------------
function TWmpPlayer.CurrentPosition(): Double;
begin
    if MediaExists() then
    begin
        Result := FPlayer.controls.currentPosition;
    end
    else
    begin
        Result := 0.0;
    end;
end;
//-----------------------------------------------------------------------------
function TWmpPlayer.MediaLength(): Double;
begin
    if MediaExists() then
    begin
        Result := FPlayer.currentMedia.duration;
    end
    else
    begin
        Result := 0.0;
    end;
end;
//-----------------------------------------------------------------------------
//  fBAύXꂽ
procedure TWmpPlayer.MediaChange(Sender: TObject; const Item: IDispatch);
var
    LMedia: IWMPMedia;
    LName: String;
    LWidth: Integer;
    LHeight: Integer;
begin
    LMedia := Item as IWMPMedia;
    LName := LMedia.name;
    if (LName = '') then
    begin
        LName := ExtractFileName(LMedia.sourceURL);
    end;
    LWidth := LMedia.imageSourceWidth;
    LHeight := LMedia.imageSourceHeight;
    if (Assigned(OnMediaChange)) then
    begin
        OnMediaChange(Self, LName, LWidth, LHeight);
    end;
end;
//-----------------------------------------------------------------------------
const
    //  PlayerPlayStateChange
    PLAY_STATE_STR: array[0..10] of string = (
        'Undefined',                'Stopped',              'Paused',
        'Playing',                  'ScanForward',          'ScanReverse',
        'Buffering',                'Waiting',              'MediaEnded',
        'Transitioning',            'Ready'
        );

    //  PlayerOpenStateChange
    OPEN_STATE_STR: array[0..21] of string = (
        'Undefined',                'PlaylistChanging',         'PlaylistLocating',
        'PlaylistConnecting',       'PlaylistLoading',          'PlaylistOpening',
        'PlaylistOpenNoMedia',      'PlaylistChanged',          'MediaChanging',
        'MediaLocating',            'MediaConnecting',          'MediaLoading',
        'MediaOpening',             'MediaOpen',                'BeginCodecAcquisition',
        'EndCodecAcquisition',      'BeginLicenseAcquisition',  'EndLicenseAcquisition',
        'BeginIndividualization',   'EndIndividualization',     'MediaWaiting',
        'OpeningUnknownURL'
        );
//-----------------------------------------------------------------------------
//  PlayStateύX
procedure TWmpPlayer.PlayStateChange(Sender: TObject; NewState: Integer);
var
    LStatus: string;
begin
    LStatus := 'Unknown';
    if (Low(PLAY_STATE_STR) <= NewState)
        and (NewState <= High(PLAY_STATE_STR)) then
    begin
        LStatus := PLAY_STATE_STR[NewState]
    end;
    if (Assigned(OnPlayStateChange)) then
    begin
        OnPlayStateChange(Self, LStatus);
    end;
end;
//-----------------------------------------------------------------------------
//  OpenStateύX
procedure TWmpPlayer.OpenStateChange(Sender: TObject; NewState: Integer);
var
    LStatus: string;
begin
    LStatus := 'Unknown';
    if (Low(OPEN_STATE_STR) <= NewState)
        and (NewState <= High(OPEN_STATE_STR)) then
    begin
        LStatus := OPEN_STATE_STR[NewState]
    end;
    if (Assigned(OnOpenStateChange)) then
    begin
        OnOpenStateChange(Self, LStatus);
    end;
end;
//-----------------------------------------------------------------------------
//  Xe[^XύX
procedure TWmpPlayer.StatusChange(Sender: TObject);
var
    LStatus: String;
begin
    if (Assigned(OnStatusChange)) then
    begin
        LStatus := FPlayer.status;
        OnStatusChange(Self, LStatus);
    end;
end;
//-----------------------------------------------------------------------------
function TWmpPlayer.GetDebugState(): Integer;
begin
    Result := FPlayer.playState;
end;

//=============================================================================
//  TRealPlayer implementation
//=============================================================================

//-----------------------------------------------------------------------------
procedure TRealPlayer.StrechToFit();
begin
    //  s
end;

//-----------------------------------------------------------------------------
//  http://hpcgi1.nifty.com/MADIA/DelphiBBS/wwwlng.cgi?print+200711/07110026.txt
//  WindowsMediaPlayer1ς̍Đ\ɂ́H
constructor TRealPlayer.Create(const AContainer: TWinControl);
begin
    inherited;
    //  CX^X쐬
    FPlayer := TRealAudio.Create(nil);
    //  ĐOFF
    FPlayer.AutoStart := False;
    //  ʃTCYςɍĐ(񂾂)
    StrechToFit();
    //  \EBhEݒ
    FPlayer.Parent := AContainer;
    FPlayer.Align := alClient;
    FPlayer.TabStop := False;
    //  Cxgݒ
    FPlayer.OnClipOpened := Self.ClipOpened;
    FPlayer.OnShowStatus := Self.ShowStatus;
    //  ςȃtO
    FFirst := True;
end;
//-----------------------------------------------------------------------------
destructor TRealPlayer.Destroy();
begin
    FPlayer.Free();
    inherited;
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.SetSource(const AValue: String);
begin
    FPlayer.Source := AValue;
    if (FFirst) then
    begin
        FFirst := False;
        //FPlayer.DoPlay();
        //  CAǏ܂H
        FPlayer.DoPlayPause();
    end;
end;
//-----------------------------------------------------------------------------
function TRealPlayer.GetSource(): String;
begin
    Result := FPlayer.Source;
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.SetMute(const AValue: Boolean); 
begin
    FPlayer.SetMute(AValue);
end;
//-----------------------------------------------------------------------------
function TRealPlayer.GetMute(): Boolean;
begin
    Result := FPlayer.GetMute();
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.SetVolume(const AValue: Integer);
begin
    FPlayer.SetVolume(AValue);
end;
//-----------------------------------------------------------------------------
function TRealPlayer.GetVolume(): Integer;
begin
    Result := FPlayer.GetVolume();
end;
//-----------------------------------------------------------------------------
function TRealPlayer.MediaExists(): Boolean;
begin
    Result := (FPlayer.CanPause()) or (FPlayer.CanPlay());
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.SetBalance(const AValue: Integer);
begin
    //  s
end;
//-----------------------------------------------------------------------------
function TRealPlayer.GetBalance(): Integer;
begin
    //  s
    Result := 0;
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.SeekFromBeginning(const Seconds: Double);
begin
    if (MediaExists() and (Seconds >= 0)) then
    begin
        FPlayer.SetPosition(Trunc(Seconds * 1000));
    end;
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.SeekFromCurrent(const Seconds: Double);
begin
    if MediaExists() then
    begin
        SeekFromBeginning(FPlayer.GetPosition() / 1000 + Seconds);
    end;
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.SeekFromEnd(const Seconds: Double);
begin
    if MediaExists() then
    begin
        SeekFromBeginning(FPlayer.GetLength() / 1000 - Seconds);
    end;
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.Play();
begin
    if MediaExists() then FPlayer.DoPlay();
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.Pause();
begin
    if MediaExists() then FPlayer.DoPause();
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.Stop();
begin
    if MediaExists() then FPlayer.DoStop();
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.OnResizeForm();
begin
    //  s
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.SetRate(const AValue: Double);
begin
    //  s
end;
//-----------------------------------------------------------------------------
function TRealPlayer.GetRate(): Double;
begin
    //  s
    Result := 1;
end;
//-----------------------------------------------------------------------------
function TRealPlayer.GetTitle(): String;
begin
    if MediaExists() then
    begin
        Result := FPlayer.GetTitle();
    end
    else
    begin
        Result := '';
    end;
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.PlayOrPause();
begin
    if (FPlayer.CanPlay()) then
    begin
        FPlayer.DoPlay();
    end
    else
    if (FPlayer.CanPause()) then
    begin
        FPlayer.DoPause();
    end;
end;
//-----------------------------------------------------------------------------
function TRealPlayer.CurrentPosition(): Double;
begin
    if MediaExists() then
    begin
        Result := FPlayer.GetPosition() / 1000;
    end
    else
    begin
        Result := 0.0;
    end;
end;
//-----------------------------------------------------------------------------
function TRealPlayer.MediaLength(): Double;
begin
    if MediaExists() then
    begin
        Result := FPlayer.GetLength() / 1000;
    end
    else
    begin
        Result := 0.0;
    end;
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.ClipOpened(Sender: TObject; const shortClipName, url: WideString);
var
    LName: String;
    LWidth: Integer;
    LHeight: Integer;
begin
    LName := FPlayer.GetTitle;
    if (LName = '') then
    begin
        LName := ExtractFileName(shortClipName);
    end;
    //  N㏉߂Ă̂Ƃ͂ǂȂ[
    LWidth := FPlayer.GetClipWidth;
    LHeight := FPlayer.GetClipHeight;

    if (Assigned(OnMediaChange)) then
    begin
        OnMediaChange(Self, LName, LWidth, LHeight);
    end;
end;
//-----------------------------------------------------------------------------
procedure TRealPlayer.ShowStatus(Sender: TObject; const statusText: WideString);
var
    LStatus: String;
begin
    if (Assigned(OnStatusChange)) then
    begin
        LStatus := Format('%d : %s', [FPlayer.GetPlayState, statusText]);
        OnStatusChange(Self, LStatus);
    end;
end;
//-----------------------------------------------------------------------------
function TRealPlayer.GetDebugState(): Integer;
begin
    Result := FPlayer.GetPlayState;
end;

end.
